%COMPARISON OF DENNIS-HOLMES AND DENNIS PARAMETER ESTIMATION
%The purpose of this program is to illustrate parameter estimation via the Dennis method which tends to work well
%with data with low non-process error versus the Dennis-Holmes method which works better when non-process error is high
%The terms "high" and "low" non-process are vague and I am still trying to work out good rules of thumb, but recognize that
%the Dennis method will always overestimate process error because it lumps process error and non-process error.  The Dennis-Holmes
%method may overestimate or underestimate.  The overestimates will be less than the Dennis method (significantly less if non-process error is high);
%but underestimates may be high for short data sets (<15 years) and high filtering if non-process error is low.
%The mu estimates via the DH method look consistently better (not as broad), but note that this is partly because 
%the Dennis method misestimates the distribution of mu when non-process error is present.

% This function estimates parameters assuming no inputs into the population; I have a correction for
%	  inputs but its not in this code as of yet.
% This function will interpolate missing data up to 2 consecutive years.
% Data file has 7 lines for each data series
% 	line 1: data name	(anything separated by a tab in the first line is considered a comment)
%	lines 2 and 3: comments; I use them for geographic and species info
%	line 4: filter to convert counts to population level count; i.e. what weight to give t, t-1, t-2, t-3 ... in that order
%	line 5: years
%	line 6: counts
%	line 7: put in a -99 for now;  this line will eventually be used for accounting for inputs into the population
%	Whitespace (ie carriage returns) between sets of data will be ignored but don't put extra whitespace within the 7 lines for each population
% Output is a tab delimited text file that can be input into excel
% Function returns a matrix of the parameter estimates; each stock is on a separate line

function paramsout=DHMethod(minyear, maxyear) %default filter length is 4

DHfilter=[1 1 1 1]; %default

%Ask whether to change default set-up
ans=input('Change default set-up (type ? for help)? ','s');
if(ans=='quit') break; end
if(ans(1)=='?') 
   disp('\n');
   disp('Default set up is no output file, no comparison');
   disp('to straight Dennis estimation and extinction level = 1.');
   disp('Type Y to change default settings.')
   disp('Type quit at anytime to get out of program.');
   disp('\n');
   ans=input('Change default set-up? ','s');
end
if(ans=='quit') break; end
if(isempty(ans) | ans(1)=='n' | ans(1)=='N') 
   makeoutputfile=0;
   DennisCompare=0;
   quasiextlevel=1;
else 
   ans=input('Make output file (default is no)? ','s');
	if(ans=='quit') break; end
	if(isempty(ans) | ans(1)=='n' | ans(1)=='N') 
      makeoutputfile=0;
   else makeoutputfile=1;end
   
   %Ask for quasi-extinction level for extinction estimates
	ans=input(['Quasi-extinction level = 1? ']);
	if(ans=='quit') break; end
	if(isempty(ans) | ans(1)=='y' | ans(1)=='Y') quasiextlevel=1;
	else quasiextlevel=ans;  end
   
   ans=input('Compare Dennis and Dennis-Holmes parameters (default is no)? ','s');
	if(ans=='quit') break; end
	if(isempty(ans) | ans(1)=='n' | ans(1)=='N') 
      DennisCompare=0;
   else DennisCompare=1;end
end

%Set up output file and print header line
if(makeoutputfile)
   fid=fopen('output.txt','w'); %tab delimited output file
   fprintf(fid,'%s\t%s\t%s\t','Stock','ESU','Species');
   fprintf(fid,'%s\t%s\t%s\t%s\t%s\t','mu','sigma2','lambda', 'lamup95', 'lamlow95');
   fprintf(fid,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t','extprob24','extprob48','extprob100', 'dec50at24', 'dec50at48','dec50at100','dec90at24','dec90at48','dec90at100');
   fprintf(fid,'%s\t%s\t%s\t%s\t%s\t%s\t','PercLPrevExt24at05','PercLPrevExt24at01','PercLPrevExt48at05', 'PercLPrevExt48at01', 'PercLPrevExt100at05','PercLPrevExt100at01');
   fprintf(fid,'%s\t%s\t%s\t%s\t%s\t%s\t','PercLPrev90dec24at05','PercLPrev90dec4at01','PercLPrev90dec48at05', 'PercLPrev90dec48at01', 'PercLPrev90dec100at05','PercLPrev90dec100at01');
   fprintf(fid,'%s\t%s\t%s\t%s\t%s\r','nq','taumax','R2', 'fitsigma', 'comments');
end

%setup output variable
paramsout=[];

%ask for and read in input file
filename=input('Name of data file: ','s');
if(filename=='quit') break; end
filestrs=textread(filename, '%s', 'whitespace', '', 'delimiter','\n'); %assigns as a cell array
%get rid of lineswith cc returns
empties=[];
for (i=1:length(filestrs)), 
   if(isempty(filestrs{i,1})) empties=[empties i]; end
end
filestrs(empties,:)=[];

%Ask for time horizon for probability of decline or extinction calculations
risktimehorizon=input('Input time horizon: ');
if(risktimehorizon=='quit') break; end

%%%%%%Go stock by stock through input file and do analyses
for (stockindex=0:7:(length(filestrs)-7)), %Each stock in file has 7 lines
   
   %Read in the next stock from the data file
   skip=0; %flag to indicate if this stock will be skipped
   dataname=filestrs{stockindex+1,1};while(isspace(dataname(end))) dataname=dataname(1:(end-1)); end
   tmp=find(double(dataname)==9); %find tabs
   if(~isempty(tmp)) dataname=dataname(1:(min(tmp)-1)); end %everything after the first tab is a comment
   esu=filestrs{stockindex+2,1};while(isspace(esu(end))) esu=esu(1:(end-1)); end
   species=filestrs{stockindex+3,1};while(isspace(species(end))) species=species(1:(end-1)); end
   popfilt=sscanf(filestrs{stockindex+4,1},'%f')';
   
   %ask if the user wants to do this stock
   ans=input(['Analyze ' dataname '? (y, n, quit)'],'s');
   if(ans=='quit') break; end
   if(isempty(ans) | ans(1)=='y' | ans(1)=='Y') %if they say yes, do all below
      
      filtlen=length(DHfilter);
      year=sscanf(filestrs{stockindex+5,1},'%f')';
      N=sscanf(filestrs{stockindex+6,1},'%f')';
      
      %Check if user input some limits on what years to analyze, otherwise use all available years in input file
      if (nargin == 0)
         minyear=min(year);
         maxyear=max(year);
      elseif (nargin==1)
         maxyear=max(year);
      end
      %%%%%Check if there is at least filtlen+8 years of data
      if (length(N)<(filtlen+8))	
         comment=['Skipped: less than ' num2str(filtlen+8) ' years of data between ' num2str(minyear) ' and ' num2str(maxyear)];
         skip=1;	
      end
      N=N(year>=minyear & year<=maxyear); %adjust by minyear and maxyear; return N=[] if no values in specified range
      year=year(year>=minyear & year<=maxyear); %adjust by minyear and maxyear
      
      %%%%%Interpolate missing values
      %Remove missing values at start and end
      while(length(N)>1 & N(1)==-99) N=N(2:end); year=year(2:end); end
      while(length(N)>1 & N(end)==-99) N=N(1:(end-1)); year=year(1:(end-1));end
      if (length(N)==1 & N(1)==-99)	
         comment='Skipped: no data for specified time period';
         skip=1;	
      end
      %Check for more than 3 missing values in a row
      if (length(N)>3 & sum(filter([1 1 1],1,N==-99)==3)>=1) 
         comment='Skipped: 3 consecutive missing values';
         skip=1;
      elseif (skip~=1 & length(N)>3) %Only do this if there is at least 3 years
         %Replace single missing values with intermediate values
         intermediate=filter([0.5 0 0.5],1,N);
         intermediate=[0 intermediate(3:end) 0];
         N(N==-99)=intermediate(N==-99);
         %Replace double missing values with intermediate values
         tmp=filter([0.5 0 0 0.5],1,N);
         intermediate=[0 tmp(4:end) -99 0 ];
         N(N<0)=intermediate(N<0);
         intermediate=[0 0 tmp(4:end) 0 ];
         N(N<0)=intermediate(N<0);
      end
      
      %%%%%Check again if there is enough data after stripping -99's; chuck if not
      if (length(N)<(filtlen+8))	
         comment=['Skipped: less than ' num2str(filtlen+8) ' years of data between ' num2str(minyear) ' and ' num2str(maxyear)];
         skip=1;	
      end
      
      %%%%%Set up running sum filter and runsum
      %runsumfilt = ones(1,filtlen);
      runsum = filter(DHfilter,1, N);
      runsum = runsum(filtlen:end);
      if(~isempty(runsum) & prod(runsum~=0)==0)
         comment='Skipped: Zeros in runsum';
         skip=1;
      end 
      
      switch skip
      case 0 % data are sufficient for anlaysis
         %%%%%Get logratios of N and runsum at tau=1
         warning off;
         runlen = length(runsum);
         Rlogratios = log(runsum(2:runlen))-log(runsum(1:(runlen-1)));
         lenN=length(N);
         Nlogratios=log(N(2:lenN)./N(1:(lenN-1)));
         tmp=1:length(Nlogratios);
         Nlogratiosyrs=tmp(~isinf(Nlogratios) & ~isnan(Nlogratios)); %indices where Nlogratios is not zero; need for trend test
         Nlogratios=Nlogratios(~isinf(Nlogratios) & ~isnan(Nlogratios)); %chuck cases where N is zero
         warning on;
         
         %%%%Do diagnostics on data; Make diagnostics figure
         %Parameter estimates are here
         diagnosticfig %script for this is in diagnosticfig.m
         
         %%%%Make risk metric figure
         riskmetricfig %script for this is in riskmetricfig.m
         
         %%%%%%Print output to file
         if(makeoutputfile)
            printoutputfile %see printoutputfile.m script
         end
         
      case 1, %Do skip this stock
         disp([dataname ' ' comment]); %Display why this one was skipped
         params=[NaN NaN NaN NaN NaN NaN NaN NaN NaN];
         if(makeoutputfile)
            fprintf(fid,'%s\t%s\t%s\t',dataname, esu, 'species');
            fprintf(fid,'%s\r',comment);
         end
      end %end of switch on whether to analyze this stock
      
      %Set up output variable which is parameter estimates for each data set
      if(isempty(paramsout))
         paramsout=params;
      else
         params=[muR muD sigma2slp sigma2np sigma2muR sigma2D dfslp lenN filtlen]; %most estimated in diagnosticfig.m
         paramsout=[paramsout; params];
      end
      
   end %if user said yes analyze this stock
   
end %end of for loop for stocks

if(makeoutputfile)
   fclose(fid);
end


